<?php

namespace FiloBlu\Flow\Model\Channel;

use Exception;
use FiloBlu\Flow\Block\Adminhtml\Channel\Edit\Tab\Config as TabConfig;
use FiloBlu\Flow\Helper\LoggerProvider;
use FiloBlu\Flow\Model\ChannelFactory;
use Monolog\Logger;

/**
 * Class Config
 * @package FiloBlu\Flow\Model\Channel
 */
class Config
{
    /**
     * @var array
     */
    public $_mappedData = [];

    /**
     * @var Logger
     */
    protected $_logger;

    /**
     * @var ChannelFactory
     */
    protected $channelFactory;

    /**
     * Config constructor.
     * @param LoggerProvider $loggerProvider
     * @param ChannelFactory $channelFactory
     */
    public function __construct(
        LoggerProvider $loggerProvider,
        ChannelFactory $channelFactory
    )
    {
        $this->_logger = $loggerProvider->getLogger();
        $this->channelFactory = $channelFactory;
    }

    /**
     * @param $name
     * @return $this
     * @throws Exception
     */
    public function load($name)
    {
        $channel = $this->channelFactory->create()->load($name, 'name');

        if (!$channel) {
            throw new Exception(__METHOD__ . "Config {$name} - Model for channel data not exist for json map");
        }

        try {
            // TODO: really necessary?
            $json_content = str_replace(["\n", "\t", "\r"], '', $channel->getData('data'));

            $mapped_config = json_decode($json_content, true);
            $this->_mappedData = $mapped_config;
            $this->_logger->info("Config {$name} - Loaded map json file");
        } catch (Exception $e) {
            throw new Exception("Config  {$name} - Fail to load map json file");
        }

        return $this;
    }

    /**
     * @return array
     */
    public function getConfigData()
    {
        return $this->_mappedData;
    }

    /**
     * @param $field
     * @return mixed|null
     */
    public function get($field)
    {
        if (isset($this->_mappedData[$field])) {
            return $this->_mappedData[$field];
        }
        return null;
    }

    /**
     * @return array
     */
    public function getFields()
    {
        return array_keys($this->_mappedData);
    }

    /**
     * @param $data
     * @return bool
     */
    public function checkData($data)
    {
        return count($data) === count($this->_mappedData);
    }

    /**
     * @return bool
     */
    public function usesMap()
    {
        if (isset($this->_mappedData['use_map'])) {
            return (bool)$this->_mappedData['use_map'];
        }

        return false;
    }

    /**
     * @return bool
     */
    public function getProcessReservedStock()
    {
        if (isset($this->_mappedData['config']['process_reserved_stock'])) {
            return (bool)$this->_mappedData['config']['process_reserved_stock'];
        }

        return false;
    }

    /**
     * @return array
     */
    public function getProcessReservedStockStatuses()
    {
        if (isset($this->_mappedData['config']['process_reserved_stock_status']) && is_array($this->_mappedData['config']['process_reserved_stock_status'])) {
            return $this->_mappedData['config']['process_reserved_stock_status'];
        }

        return [];
    }

    /**
     * @return bool
     */
    public function getDiscardNoManagedStock()
    {
        if (isset($this->_mappedData['config']['discard_no_managed'])) {
            return (bool)$this->_mappedData['config']['discard_no_managed'];
        }

        return false;
    }

    /**
     * @return bool
     */
    public function getIndexerStrategy()
    {
        if (isset($this->_mappedData['config']['indexer_strategy'])) {
            return $this->_mappedData['config']['indexer_strategy'];
        }

        return TabConfig::INDEXER_STRATEGY_FULL;
    }

    /**
     * @return bool
     */
    public function getPreorderDiscard()
    {
        if (isset($this->_mappedData['config']['preorder_discard'])) {
            return (bool)$this->_mappedData['config']['preorder_discard'];
        }

        return false;
    }

    /**
     * @return string
     */
    public function getImageMediaGallery()
    {
        if (isset($this->_mappedData['config']['image_media_gallery'])) {
            return (bool)$this->_mappedData['config']['image_media_gallery'];
        }

        return 'media_gallery';
    }

    /**
     * @return string|null
     */
    public function getPreorderAttributeCode()
    {
        if (isset($this->_mappedData['config']['preorder_attribute_code'])) {
            return $this->_mappedData['config']['preorder_attribute_code'];
        }

        return '';
    }

    /**
     * @return null|string
     */
    public function getPreorderAttributeValue()
    {
        if (isset($this->_mappedData['config']['preorder_attribute_value'])) {
            return $this->_mappedData['config']['preorder_attribute_value'];
        }

        return null;
    }
}
